@@ -20,11 +20,11 @@
 import SocketServer
 import re
 import os, stat, os.path, mimetypes
-from pygopherd import handlers, GopherExceptions, logger
+from pygopherd import handlers, GopherExceptions, logger, gopherentry
 from pygopherd.handlers import HandlerMultiplexer
 
 class BaseGopherProtocol:
-    """Skeleton protocl -- includes commonly-used routines."""
+    """Skeleton protocol -- includes commonly-used routines."""
     def __init__(self, request, server, requesthandler, rfile, wfile, config):
         """Parameters are:
         request -- the raw request string.
@@ -46,6 +46,7 @@ class BaseGopherProtocol:
         self.requesthandler = requesthandler
         self.requestlist = requestparts
         self.searchrequest = None
+        self.handler = None
 
         selector = requestparts[0]
 
@@ -62,6 +63,7 @@ class BaseGopherProtocol:
         return 0
 
     def log(self, handler):
+        """Log a handled request."""
         logger.log("%s [%s/%s]: %s" % \
                    (self.requesthandler.client_address[0],
                     re.search("[^.]+$", str(self.__class__)).group(0),
@@ -75,7 +77,10 @@ class BaseGopherProtocol:
             self.log(handler)
             self.entry = handler.getentry()
             handler.prepare()
-            handler.write(self.wfile)
+            if handler.isdir():
+                self.writedir(self.entry, handler.getdirlist())
+            else:
+                handler.write(self.wfile)
         except GopherExceptions.FileNotFound, e:
             self.filenotfound(str(e))
         except IOError, e:
@@ -85,21 +90,66 @@ class BaseGopherProtocol:
     def filenotfound(self, msg):
         self.wfile.write("3%s\t\terror.host\t1\r\n" % msg)
 
-    def renderobjinfo(self, entry):
-        """Renders an object's info according to the protocol.  Returns
-        a string.  A gopher0 server, for instance, would return a dir line."""
-        pass
-
     def gethandler(self):
         """Gets the handler for this object's selector."""
-        return HandlerMultiplexer.getHandler(self.selector, self.searchrequest,
+        if not self.handler:
+            self.handler = HandlerMultiplexer.getHandler(self.selector, self.searchrequest,
                                                self, self.config)
+        return self.handler
+
+    def writedir(self, entry, dirlist):
+        """Called to render a directory.  Generally called by self.handle()"""
+
+        startstr = self.renderdirstart(entry)
+        if startstr != None:
+            self.wfile.write(startstr)
+
+        abstractopt = self.config.get("pygopherd", "abstract_entries")
+        doabstracts = abstractopt == 'always' or \
+                      (abstractopt == 'unsupported' and
+                       not self.groksabstract())
+
+        if self.config.getboolean("pygopherd", "abstract_headers"):
+            self.wfile.write(self.renderabstract(entry.getea('ABSTRACT', '')))
+
+        for direntry in dirlist:
+            self.wfile.write(self.renderobjinfo(direntry))
+            if doabstracts:
+                abstract = self.renderabstract(direntry.getea('ABSTRACT'))
+                if abstract:
+                    self.wfile.write(abstract)
+
+        endstr = self.renderdirend(entry)
+        if endstr != None:
+            self.wfile.write(endstr)
+
+    def renderabstract(self, abstractstring):
+        if not abstractstring:
+            return ''
+        retval = ''
+        for line in abstractstring.splitlines():
+            absentry = gopherentry.getinfoentry(line, self.config)
+            retval += self.renderobjinfo(absentry)
+        return retval
 
     def renderdirstart(self, entry):
         """Renders the start of a directory.  Most protocols will not need
-        this.  Exception might be HTML.  Returns None if not needed."""
+        this.  Exception might be HTML.  Returns None if not needed.
+        Argument should be the entry corresponding to the dir itself."""
         return None
 
     def renderdirend(self, entry):
         """Likewise for the end of a directory."""
         return None
+
+    def renderobjinfo(self, entry):
+        """Renders an object's info according to the protocol.  Returns
+        a string.  A gopher0 server, for instance, would return a dir line.
+        MUST BE OVERRIDDEN."""
+        return None
+
+    def groksabstract(self):
+        """Returns true if this protocol understands abstracts natively;
+        false otherwise."""
+        return 0
+    
